const express = require('express');
const { MongoClient, ObjectId } = require('mongodb');
const app = express();
const port = process.env.PORT || 3000;

const url = 'mongodb://localhost:27017';
const dbName = 'test';
let db;

MongoClient.connect(url,{ useNewUrlParser: true, useUnifiedTopology: true })
    .then(client => {
        console.log('Połączono z baza MongoDB');
        db = client.db(dbName);
    })
    .catch(error => console.error('Błąd połączenia z baza MongoDB:', error));

app.use(express.urlencoded({ extended: true }));
app.use(express.json());

app.listen(port, () => {
  console.log(`Serwer działa na porcie ${port}`);
});





app.get('/',(req,res)=>{
  res.sendFile(__dirname+'/index.html');
});


app.get('/usun',async(req,res)=>{
  const produktId = req.query.id;
  try{
    await db.collection('produkty').deleteOne({ _id: new ObjectId(produktId) });;
    res.redirect('/wyswietl');
  }
  catch(e){
    console.log("Error: "+e);
  }
});


app.post('/edytuj', async (req, res) => {
  try {
    const edytowanyProdukt = {
      produkt_id: req.body.produkt_id,
      nazwa: req.body.nazwa,
      opis: req.body.opis,
      cena: parseFloat(req.body.cena),
      ilosc_na_stanie: parseInt(req.body.ilosc_na_stanie),
      kategoria: req.body.kategoria
    };
    const produktId = req.body.pid;
    await db.collection('produkty').updateOne({ _id: new ObjectId(produktId) }, { $set: edytowanyProdukt });
    res.redirect('/wyswietl');
  } catch (err) {
    console.error('Błąd podczas edytowania produktu:', err);
    res.status(500).send("Wystąpił błąd podczas edytowania produktu w bazie.");
  }
});


app.get('/edytuj', async (req, res) => {
  const retId = req.query.id;
  let client;

  try {
    var html = `
    <html lang="pl">
    <head>
    <meta charset="UTF-8" />
    <title>Edycja produktu</title>
    </head>
    <body>
        <h1>Edytuj Produkt</h1>    `;

    const produkt = await db.collection('produkty').findOne({ _id: new ObjectId(retId) });

    if (!produkt) {
      res.status(404).send("Produkt nie znaleziony");
      return;
    }
    html += `
      <form action="/edytuj" method="post">
        <input name="pid" type="hidden" value="${produkt._id}"/>
        <label for="produkt_id">ProduktId:</label><br>
        <input value="${produkt.produkt_id}" type="text" id="produkt_id" name="produkt_id" ><br>
        <label for="nazwa">Nazwa:</label><br>
        <input value="${produkt.nazwa}" type="text" id="nazwa" name="nazwa" ><br>
        <label for="opis">Opis:</label><br>
        <input value="${produkt.opis}" type="text" id="opis" name="opis"><br>
        <label for="cena">Cena:</label><br>
        <input value="${produkt.cena}" type="number" id="cena" name="cena"><br>
        <label for="ilosc_na_stanie">Ilość na stanie:</label><br>
        <input value="${produkt.ilosc_na_stanie}" type="number" id="ilosc_na_stanie" name="ilosc_na_stanie"><br>
        <label for="kategoria">Kategoria:</label><br>
        <input value="${produkt.kategoria}" type="text" id="kategoria" name="kategoria"><br><br>
        <input type="submit" value="Edytuj Produkt">
      </form>`;

    html += `
      </body>
    </html>
    `;

    res.send(html);
  } catch (e) {
    console.log('Error:', e);
    res.status(500).send("Wystąpił błąd podczas edytowania produktu.");
  }
});

app.get('/dodaj', (req, res) => {
  res.sendFile(__dirname + '/dodaj.html');
});

app.post('/dodaj', async (req, res) => {
  try {
    const maxid = db.collection('produkty').find().sort({ produkt_id: -1 }).limit(1);
    const maxid2 = await maxid.next();
    const maxidf = maxid2.produkt_id + 1;
    const nowyProdukt = {
      produkt_id: maxidf,
      nazwa: req.body.nazwa,
      opis: req.body.opis,
      cena: parseFloat(req.body.cena),
      ilosc_na_stanie: parseInt(req.body.ilosc_na_stanie),
      kategoria: req.body.kategoria
    };

    await db.collection('produkty').insertOne(nowyProdukt);
    res.redirect('/wyswietl');
  } catch (err) {
    console.error('Błąd podczas dodawania nowego produktu:', err);
    res.status(500).send("Wystąpił błąd podczas dodawania nowego produktu do bazy.");
  }
});


app.get('/aktualizuj', async(req,res)=>{
  try{
    let html = ``;
    const kategorie = await db.collection('produkty').find({}).toArray();
    
    var cat_list = [];
    kategorie.forEach((kategoria)=>{
if(cat_list.indexOf(kategoria.kategoria)==-1)cat_list.push(kategoria.kategoria);
    });

    html+=`
    <h1>Aktualizacja ceny z okreslonej kategorii</h1>
    <form method='post' action='/aktualizuj'>
    <label>Wybierz kategorie: <select name='kategoria'>
    `;
    cat_list.forEach((cat)=>{
      html+=`<option value="${cat}">${cat}</option>`;
    });
    html+=`
    </select></label></br>
    <label>Podaj wartosc w [%] <input type='number' name='ilosc'/></label><br/>
    <input type='submit' value='Aktualizuj'>
    </form>
    `;
    res.send(html);
  }
  catch(e){
    console.log(e);
  }
});
app.post('/aktualizuj',async (req,res)=>{
  try{
const kategoria = req.body.kategoria,ilosc = req.body.ilosc;

var produkty = await db.collection('produkty').find({kategoria:{$eq:kategoria}}).toArray();

produkty.forEach(async (produkt)=>{
var nowa_wartosc = produkt.cena*ilosc*0.01;
await db.collection('produkty').updateOne({_id: new ObjectId(produkt._id)},{$set:{'cena':nowa_wartosc}});
});
res.redirect('/wyswietl');
  }
  catch(e){
    console.log(e);
  }
});

app.get('/wyswietl', async (req, res) => {
  try {
    const produkty = await db.collection('produkty').find({}).toArray();
    let html = `
      <html>
      <head>
        <title>Lista Produktów</title>
        <style>
          table { width: 100%; border-collapse: collapse; }
          table, th, td { border: 1px solid black; }
          th, td { padding: 15px; text-align: left; }
        </style>
      </head>
      <body>
        <h1>Lista Produktów</h1>
        <table>
          <tr>
            <th>Produkt_ID</th>
            <th>Nazwa</th>
            <th>Opis</th>
            <th>Cena</th>
            <th>Ilość na stanie</th>
            <th>Kategoria</th>
            <th>Operacje</th>
          </tr>`;

    produkty.forEach(produkt => {
      html += `
        <tr>
          <td>${produkt.produkt_id}</td>
          <td>${produkt.nazwa}</td>
          <td>${produkt.opis}</td>
          <td>${produkt.cena}</td>
          <td>${produkt.ilosc_na_stanie}</td>
          <td>${produkt.kategoria}</td>
          <td>
            <form action='/edytuj' method='get'>
              <input type="hidden" name="id" value="${produkt._id}">
              <input type='submit' value='Edytuj'/>
            </form>
            <form action='/usun' method='get'>
              <input type='hidden' name='id' value='${produkt._id}'/>
              <input type='submit' value='Usuń'/>
            </form>                    
          </td>
        </tr>`;
    });
    html += ` </table></body></html>`;
    res.send(html);
  } catch (err) {
    console.error(err);
  } 
});


